package simulation;

public class ComponentRoute {
    public static final int SPEED = 1;

    private final Component component;
    private final Building outputBuilding;
    private final int directionX;
    private final int directionY;

    private int x;
    private int y;

    public ComponentRoute(Component component, Building outputBuilding, int x, int y) {
        this.component = component;
        this.outputBuilding = outputBuilding;
        this.x = x;
        this.y = y;

        directionX = normalizedDirection(x, outputBuilding.getX());
        directionY = normalizedDirection(y, outputBuilding.getY());
    }

    private int normalizedDirection(int from, int to) {
        return Integer.compare(to, from);
    }

    public void updatePosition() {
        x += directionX;
        y += directionY;
    }

    public void sendToOutputBuilding() {
        outputBuilding.processInput(component);
    }

    public boolean isTransitFinished() {
        return x == outputBuilding.getX() && y == outputBuilding.getY();
    }

    public Component getComponent() {
        return component;
    }

    public Building getOutputBuilding() {
        return outputBuilding;
    }

    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }
}
